#!R

## parameters

args <- commandArgs(trailingOnly=TRUE)
if(length(args) < 6) stop('Insufficient arguments supplied. At least 6 arguments expected. See master_command_file for example')
outputpdfile <- paste(args[1], '.pdf', sep='')
FXcloserates.file <- args[2]
if(FXcloserates.file==0) FXcloserates.file <- NULL
SimRates.file <- args[3]
if(nchar(SimRates.file)<4) SimRates.file <- NULL # if SimRates.file set to less than 4 characters cannot be valid file
Seed <- as.numeric(args[4])
numResultsRDataFiles <- as.numeric(args[5])
hist_aggregate <- args[6]

if(length(args)>6){
date_range_restrict_end <- args[8]
date_range_restrict_start <- args[7]
date_range_restrict <- as.Date(c(date_range_restrict_end, date_range_restrict_start), format='%Y-%m-%d')
} else date_range_restrict <- NULL

FXconsistenttransactions.file <- NULL

# Obtain FX / ON rates data and merge together (accounting for settlement lags)


set.seed(Seed)
if(!is.null(FXcloserates.file)){
	print('Loading actual daily USD/CAD FX spot rates')
	load(FXcloserates.file) # loading FX close rates data of form: Ticker (USDCAD), datetime, Close (rate)
					# colnames = Ticker, datetime.Can, Close (rate)
						# datetime.Can is either in POXIX or Date format						
	library(plyr)
	day <- as.Date(canFX$datetime.CAN)
	canFX <- data.frame(canFX, day)
	canFXdaily <- ddply(canFX, .(day), function(x){
		max_rate_d <- max(x$Close)
		min_rate_d <- min(x$Close)
		total_diff <- max_rate_d - min_rate_d
		mean_rate_d <- mean(x$Close)
		median_rate_d <- median(x$Close)
	        sd_rate_d <- sd(x$Close)
        	Y <- cbind(min_rate_d, max_rate_d, mean_rate_d, median_rate_d, sd_rate_d, total_diff)
	})
	BoCinterestRates <- read.table('Inputs/BoCinterestRates_012005-112013.csv', sep=',', skip=44, header=T, na.strings=c('Not available', 'Bank holiday'))
	BoCband <- BoCinterestRates[,c(1,2,6,7)] # date, target, low, high
	suppressWarnings(BoCband[,c(2:4)] <- apply(BoCband[,c(2:4)], 2, as.numeric))
	BoCband[,1] <- as.Date(BoCband[,1], format='%d/%m/%Y')

	# merging the contemporaneous overnight rates / with T+1 FX rates
	canFXdaily <- merge(canFXdaily, BoCband, by.y='Date', by.x='day')
	lmatrix <- matrix(0,nrow(canFXdaily), nrow(canFXdaily))
	lmatrix[-1, -nrow(canFXdaily)] <- diag(nrow(canFXdaily) -1)
	FXrates <- as.matrix(canFXdaily[,c('min_rate_d', 'max_rate_d', 'median_rate_d')])
	lagged_FX <- lmatrix %*% FXrates

	canFXinterday <- data.frame(canFXdaily, lagged_FX)
	canFXinterday[1,c(11:13)] <- rep(NA_real_,3)

	names(canFXinterday) <- c('date', 'Cmin_rate_d', 'Cmax_rate_d', 'Cmean_rate_d', 'Cmedian_rate_d', 'Csd_rate_d', 'Ctotal_diff', 'target', 'lowerb', 'upperb', 'min_rate_d', 'max_rate_d', 'median_rate_d')


} else {
	print('No FX data found, using simulated data for illustration purposes')
	
	load(SimRates.file)
	# get daily max / min FX spot rates lagged by T+1 settlement for Canada
	
	min_FXspot_rates <- ratesData$FXrate + rnorm(nrow(ratesData), mean=0, sd=0.01)
	max_FXspot_rates <- min_FXspot_rates + rnorm(nrow(ratesData),mean=0.005, sd=0.0005) 
	min_ON_rate <- ratesData$ONrate - 0.25
	max_ON_rate <- ratesData$ONrate + 0.25
	FXdata <- data.frame(ratesData$date, min_FXspot_rates, max_FXspot_rates, ratesData$ONrate, min_ON_rate, max_ON_rate)

	canFXinterday <- FXdata

	names(canFXinterday) <- c('date', 'min_rate_d', 'max_rate_d', 'target', 'lowerb', 'upperb')
	canFXinterday[,1] <- as.Date(as.character(canFXinterday[,1]), format='%Y%m%d')

}

# Obtaining the FX-consistent transactions from the payments data

if(!is.null(FXconsistenttransactions.file)){
	load(FXconsistenttransactions.file) # loading pre-generated USD CAD consistent transaction values
        CADfx.u <- unique(CADfx.ON.6am[,c(1,3,4)])
} else {
	# Constructing FX-consistent transactions from the payments data
	# i.e. interpolate the range of spot-rates traded within the day and multiply by round value levels
		# make a sequence of values, 
	print('Constructing FX consistent transactions')
	million <- 10^6
	#round_values <- c(1:5, 7,10)*million
	round_values <- 1:10*million
	# FX-consistent transactions
	pip <- 0.0001
	FXrateRange.daily <- do.call(rbind, lapply(1:nrow(canFXinterday), function(x){
			rate_range <- NULL
			try(rate_range <- seq(as.numeric(canFXinterday$min_rate_d[x]),as.numeric(canFXinterday$max_rate_d[x]), pip), silent=T)
			if(is.null(rate_range)) rate_range <- 0
			FXconsistentValues.day <- rate_range%*%t(round_values)
			date.day <- canFXinterday[x,1]
			output <- data.frame(rep(date.day, nrow(FXconsistentValues.day)), FXconsistentValues.day)
			return(output)
	}))

	names(FXrateRange.daily)[1] <- 'date'

	# transform from wide to long form
	CADfx.u <- unique( do.call(rbind, lapply(2:(ncol(FXrateRange.daily)), function(x) as.matrix(FXrateRange.daily[,c(1,x)]))))
	dates <-  as.Date(CADfx.u[,1], format='%Y-%m-%d')
	CADfx <- as.numeric(CADfx.u[,2])
	CADfx.u <- data.frame(dates, CADfx)
}



# restricting to date range in paper
if(!is.null(date_range_restrict)) canFXinterday <- canFXinterday[which(canFXinterday[,1] < date_range_restrict[1] & canFXinterday[,1] >=date_range_restrict[2]),]


ond <- 365
smallest_FXint <- 0.0001/canFXinterday$max_rate_d*ond*100
canFXinterday$FXint_lb <- smallest_FXint


# or (SpotRate*1m USD - 1m Cad) / 1m Cad


FX_int_twoway_Cbase_min <- (canFXinterday$max_rate_d - 1)*ond*100
FX_int_twoway_Cbase_max <- (canFXinterday$min_rate_d - 1)*ond*100


# Identify days where rates are `near' parity
	# i.e. days in which the max rate exceeds parity and the min rate falls below parity

twoway_FX_indicator <- rep(-5, length(canFXinterday[,1]))
twoway_FX_indicator[which(apply(cbind(FX_int_twoway_Cbase_min, FX_int_twoway_Cbase_max),1, max) > 0 & apply(cbind(FX_int_twoway_Cbase_min, FX_int_twoway_Cbase_max),1,min) < 0)] <- 0


# Merge Spurious matches with the FX-consistent transactions

print('loading spurious matches')
Full_str <- 'BaselineTypeI'
i <- 1
Matches <- NULL
pfdrs <- vector('list', 30)
for(i in 1:numResultsRDataFiles){
        load(paste('Results/', Full_str,'_', i, '.RData', sep='')) -> a
        typeI <- eval(parse(text=a))
        matches <- do.call(rbind, typeI$raw)
        pfdrs[[i]] <- typeI$tau
        if(is.null(Matches)) Matches <- matches else Matches <- rbind(Matches, matches)
}

# if the Results files were too large and needed to be split into two..
if(exists((paste('Results/', Full_str,'B_',i,'.RData', sep='')))){
	print('TypeImatch files were split into A/B. Loading/appending B files')
	for(i in 1:numResultsRDataFiles){
		suppressWarnings(load(paste('Results/', Full_str,'B_',i,'.RData', sep='')) -> a)
		typeI <- eval(parse(text=a))
		matches <- do.call(rbind, typeI$raw)
		pfdrs[[i+15]] <- typeI$tau
		if(is.null(Matches)) Matches <- matches else Matches <- rbind(Matches, matches)
	}
}

Matches.Su <- unique(Matches[,c(1:10)])

names(Matches.Su) <- c('date', 'time', 'value', 'sender', 'receiver', 'tranche', 'ONrate', 'jumbo', 'misc', 'transnum')
FullS.merge <- merge(Matches.Su, CADfx.u ,by.x=c('date', 'value'), by.y=c('dates', 'CADfx'))
FullShist <- NULL
if(nrow(FullS.merge)>0) FullS.merge.U <- unique(FullS.merge[,c(1:ncol(FullS.merge))]) else FullS.merge.U <- FullS.merge

suppressWarnings(try(FullShist <- hist(FullS.merge.U$date, hist_aggregate, format='%Y-%m-%d', freq=F, col='red', main='', xlab='',  axes=F, ylab=''), silent=T))


# Plot

print('plotting Interest and FX rate configuration with FX consistent spurious matches')
require(zoo)
num.obs <- nrow(canFXinterday)
pdf(outputpdfile)
plot(canFXinterday[,1], canFXinterday$FXint_lb, type='l', col='blue', ylim=c(-2,6), xlab='Date', ylab='Interest rate (%)', lwd=3)
lines(canFXinterday[,1],canFXinterday$upperb+0.25, col='black')

# highlighting range where FX lower bound falls within algorithm interest rate filter
x <- c(canFXinterday[1,1], canFXinterday[,1], rev(canFXinterday[,1]), canFXinterday[1,1])
y <- c(-2, canFXinterday$upperb+0.25,  -2, rep(-2, nrow(canFXinterday)))
y <- na.approx(y)
polygon(x, y, col='red', border=NA) # highlight the range below upper bound interest rate of filter
yw <- c(canFXinterday$lowerb[1] -0.25,canFXinterday$lowerb -0.25, rep(-2 -0.001, length(canFXinterday[,1]) + 1))
yw <- na.approx(yw)
polygon(x, yw, col='white', border=NA, lwd=3) # whiteout the range below the lower bound interest rate of algorithm 
lb_greater_FXlb <- which(canFXinterday$lowerb-0.25 > canFXinterday$FXint_lb) # 
ub_less_FXlb <- which(canFXinterday$upperb+0.25 < canFXinterday$FXint_lb)


# Whitenening out the upper bound interest rate below the FX upper bound 
White.breakpoints <- which(diff(ub_less_FXlb) > 1)
# for each break point
if(length(White.breakpoints)>0){
	  if(length(White.breakpoints)==1){
	        w.xrange <- seq(ub_less_FXlb[1], ub_less_FXlb[White.breakpoints[1]] )

                 w.canFXinterday<- canFXinterday[w.xrange,]
                 xw <- c(w.canFXinterday[1,1],w.canFXinterday[,1], rev(w.canFXinterday[,1]),w.canFXinterday[1,1])
                 yw <-  c(w.canFXinterday$lowerb[1]-0.25, w.canFXinterday$upperb + 0.25, rev(w.canFXinterday$lowerb) -0.25, w.canFXinterday$lowerb[1])
                 yw <- na.approx(yw)
                 polygon(xw, yw, col='white', border=NA, lwd=3)

  		 i <- 1
	} else {

		for(i in 2:length(White.breakpoints)){
			# 
			w.xrange <-seq( ub_less_FXlb[White.breakpoints[i-1] +1],ub_less_FXlb[ White.breakpoints[i]])
			w.canFXinterday<- canFXinterday[w.xrange,]
			xw <- c(w.canFXinterday[1,1],w.canFXinterday[,1], rev(w.canFXinterday[,1]),w.canFXinterday[1,1])
			yw <-  c( w.canFXinterday$upperb[1]+0.25,  w.canFXinterday$FXint_lb, rev( w.canFXinterday$lowerb) -0.25,  w.canFXinterday$lowerb[1]-0.25)
			yw <- na.approx(yw)
			polygon(xw, yw, col='white', border=NA,lwd=3)
		}
	} # filling in white gap from last breakpoint to end of index
	 w.xrange <-seq( ub_less_FXlb[White.breakpoints[i] + 1],ub_less_FXlb[length(ub_less_FXlb)])
	 w.canFXinterday<- canFXinterday[w.xrange,]
	 xw <- c(w.canFXinterday[1,1],w.canFXinterday[,1], rev(w.canFXinterday[,1]),w.canFXinterday[1,1])
	 yw <-  c( w.canFXinterday$upperb[1]+0.25,  w.canFXinterday$FXint_lb, rev( w.canFXinterday$lowerb) -0.25,  w.canFXinterday$lowerb[1]-0.25)
	 yw <- na.approx(yw)
	 polygon(xw, yw, col='white', border=NA, lwd=3)	
} else {
	w.xrange <- ub_lesser_FXlb
	 w.canFXinterday<- canFXinterday[w.xrange,]
	
	 xw <- c(w.canFXinterday[1,1],w.canFXinterday[,1], rev(w.canFXinterday[,1]),w.canFXinterday[1,1])
	 yw <-  c( w.canFXinterday$upperb[1]+0.25,  w.canFXinterday$FXint_lb, rev( w.canFXinterday$lowerb) -0.25,  w.canFXinterday$lowerb[1]-0.25)
	 yw <- na.approx(yw)
	 polygon(xw, yw, col='white', border=NA, lwd=3)
}

# Whitenening out the lower bound interest rate above the FX upper bound 

White.breakpoints<- which(diff(lb_greater_FXlb) > 1)
# for each break point

if(length(White.breakpoints)>0){
	if(length(White.breakpoints)==1){
		w.xrange <- seq(lb_greater_FXlb[1], lb_greater_FXlb[White.breakpoints[1]])
		 w.canFXinterday<- canFXinterday[w.xrange,]
		 xw <- c(w.canFXinterday[1,1],w.canFXinterday[,1], rev(w.canFXinterday[,1]),w.canFXinterday[1,1])
	 	 yw <-  c(w.canFXinterday$lowerb[1]-0.25, w.canFXinterday$upperb + 0.25, rev(w.canFXinterday$lowerb) -0.25, w.canFXinterday$lowerb[1])
		 yw <- na.approx(yw)
		 polygon(xw, yw, col='white', border=NA)

	} else {
		for(i in 2:length(White.breakpoints)){
			# 
			w.xrange <-seq( lb_greater_FXlb[White.breakpoints[i-1] +1],lb_greater_FXlb[ White.breakpoints[i]-1])
			w.canFXinterday<- canFXinterday[w.xrange,]
			xw <- c(w.canFXinterday[1,1],w.canFXinterday[,1], rev(w.canFXinterday[,1]),w.canFXinterday[1,1])
			yw <-  c(w.canFXinterday$lowerb[1]-0.25, w.canFXinterday$upperb + 0.25, rev(w.canFXinterday$lowerb) -0.25, w.canFXinterday$lowerb[1])

			yw <- na.approx(yw)
			polygon(xw, yw, col='white', border=NA)
		}

		 w.xrange <-seq( ub_less_FXlb[White.breakpoints[i]],lb_greater_FXlb[length(lb_greater_FXlb)])
		 w.canFXinterday<- canFXinterday[w.xrange,]
		 xw <- c(w.canFXinterday[1,1],w.canFXinterday[,1], rev(w.canFXinterday[,1]),w.canFXinterday[1,1])
		 yw <-  c(w.canFXinterday$lowerb[1]-0.25, w.canFXinterday$upperb + 0.25, rev(w.canFXinterday$lowerb) -0.25, w.canFXinterday$lowerb[1])
		 yw <- na.approx(yw)
	         polygon(xw, yw, col='white', border=NA)
	}	

} else {
	         w.xrange <-lb_greater_FXlb
		 if(length(w.xrange)>0){
			 w.canFXinterday<- canFXinterday[w.xrange,]
			 xw <- c(w.canFXinterday[1,1],w.canFXinterday[,1], rev(w.canFXinterday[,1]),w.canFXinterday[1,1])
			 yw <-  c(w.canFXinterday$lowerb[1]-0.25, w.canFXinterday$upperb + 0.25, rev(w.canFXinterday$lowerb) -0.25, w.canFXinterday$lowerb[1])
			 yw <- na.approx(yw)
			 polygon(xw, yw, col='white', border=NA)
		}
}

# cleaning up the edges of the plot
abline(v=canFXinterday[num.obs,1], col='white', lwd=3)
abline(v=canFXinterday[1,1], col='white', lwd=3)
lines(canFXinterday[,1], canFXinterday$FXint_lb, col='blue')
lines(canFXinterday[,1],canFXinterday$upperb+0.25, col='black', lty=2)
lines(canFXinterday[,1], canFXinterday$lowerb-0.25, col='black', lty=4)
points(canFXinterday[,1], twoway_FX_indicator, col='gold1', cex=1, pch=3)
lines(canFXinterday[,1], canFXinterday$FXint_lb, type='l', col='blue',lwd=3)
box('plot')
legend('topright', c('FX inferred rate lower bound', 'Upper/Lower bound interest threshold', 'FX inferred rate in interest threshold', 'Near parity USD-CAD spot rates'), lty=c(1,3,1, 4), col=c('blue', 'black', 'red','gold1') , lwd=(c(2,1,1,1)), cex=0.8)
par(new=plot)
suppressWarnings(ymax <- max(FullShist$density)*2.5)
if(nrow(FullS.merge.U)>0){ 
	hist(FullS.merge.U$date, hist_aggregate, format='%Y-%m-%d', freq=F, col='red', main='', xlab='', ylim=c(0,ymax), axes=F, ylab='') 
	axis(4, at=pretty(FullShist$density, n=5), labels=F)
} else print('no FX-consistent spurious matches found - thus no barplot added')
dev.off()


