#! R
#################################################
# This generates the intraday time-value distribution (Figure 2 Rempel (2016))
#
# Inputs: 
#	1) Furfine Loan data in csv format
#	2) Faux Loan data in csv format
#	3) Output pdf file name (without extension)
#	optional additional inputs:
#	4) max ylim for mean value of payments
#	4-7) max ylim for each of unique loans, all loans, faux loans, non-unique loans
#	8) faux loans number alternative access
# Output:
#	four pdfs corresponding to: uniqueloans, all loans, fauxloans, duplicate loans
#		e.g. outputpdfname_UniqueLoans.pdf 
##################################################

args <- commandArgs(trailingOnly=T)
furfInput <- args[1]
FauxInput <- args[2]
OutputFile <- args[3]
ymax_base <- 300
Nymax_base <- 13
Nymax <- NULL
Nymin_base <- 1
Nymax.l <- rep(Nymax_base, 4)
Nymin.l <- rep(Nymin_base, 4)
ymax.l <- rep(ymax_base, 4)

print(paste('Number of arguments supplied is ', length(args)))
if(length(args) > 3){
	ymax <- as.numeric(args[4])
	if(length(args) > 4){
		if(length(args) < 7) stop('Number of arguments invalid')
		ymax.l <- as.numeric(c(args[4], args[5], args[6], args[7]))
		if(length(args) ==8) Nymax.l[3] <- as.numeric(args[8])
		if(length(args) == 9) Nymin <- as.numeric(args[9])
		if(length(args) > 9){
			# ylimits set for every plot
			Nymax.l <- as.numeric(c(args[8], args[9], args[10], args[11]))
			if(length(args) > 11){
				Nymin.l <- as.numeric(c(args[12], args[13], args[14], args[15]))
			}
		} 
	} else ymax.l <- rep(ymax,4)
} 


furfdat <- read.csv(furfInput, header=F)
#furfdat <- read.csv('FurfineDataLastQuarter.csv', header=F)
colnames(furfdat) <-  c('date', 'time', 'value', 'sender', 'receiver', 'tranche', 'ONrate', 'jumbo', 'misc', 'transnum', 'date_r', 'time_r', 'value_r', 'sender_r', 'receiver_r', 'tranche_r', 'ONrate_r', 'jumbo_r', 'misc_r', 'transnum_r', 'furf_rate', 'mltpl')

faux <- read.csv(FauxInput, header=F)
#faux <- read.csv('FauxLoansLastQuarter.csv', header=F)
colnames(faux) <-  c('date', 'time', 'value', 'sender', 'receiver', 'tranche', 'ONrate', 'jumbo', 'misc', 'transnum', 'date_r', 'time_r', 'value_r', 'sender_r', 'receiver_r', 'tranche_r', 'ONrate_r', 'jumbo_r', 'misc_r', 'transnum_r', 'furf_rate', 'mltpl')


furfdat <- as.data.frame(furfdat)
faux <- as.data.frame(faux)

ufurfdat <- furfdat[which(furfdat$mltpl==0),]
dupfurfdat <- furfdat[which(furfdat$mltpl>0),]

cuts <- sapply(6:18, function(x){
        #sq <- seq(0, 45, 15)
        sq <- c('00', '15','30', '45')
#        sq <- c(paste('0', seq(0, 9), sep=''), as.character(seq(10,59)))
        times <- as.numeric(paste(x, sq, '00', sep=''))
})

IntradaySumF <- function(x, cts=cuts){
	x_time.f <- cut(x$time, breaks=cts, labels=cts[-c(1)])
	x.l <- split(x, x_time.f)
	x.mv <- sapply(x.l, function(z) mean(z$value)/10^6)
	x.n <- sapply(x.l, function(z){
		y <- split(z, as.factor(z$date))
		n <- mean(sapply(y, function(z) if(class(z)=='data.frame') nrow(z)))
	}) 
	out <- list(x.mv, x.n)
}

DAT <- list(ufurfdat, furfdat, faux, dupfurfdat)
OUT <- vector('list', length(DAT))

for(i in 1:length(DAT)){
	OUT[[i]] <- IntradaySumF(DAT[[i]])
}



require(zoo)
blu1 <- adjustcolor('blue', 0.5)
#pdf('PELB_ValueTimeDistribution.pdf')
out_extensions <- c('UniqueLoans', 'AllLoans', 'FauxLoans', 'DuplicateLoans')

for(f in 1:length(out_extensions)){
	print(f)
	out_file <- paste(OutputFile, out_extensions[f], '.pdf', sep='')
	pdf(out_file)
	meanv <- OUT[[f]][[1]]
	nv <- OUT[[f]][[2]] 
	ymax <- ymax.l[f]
	yymax <- Nymax.l[f]
	yymin <- Nymin.l[f]
	meanv[which(is.na(meanv))] <- 0
	nv[which(is.na(nv))] <- 0
#	print(paste('Ymax', ymax))
        bp <- barplot(meanv, col='red', xaxt='n', add=F, yaxt='n', xlab='Time (15 minute intervals)', ylab='Mean value in millions', ylim=c(0, ymax), xlim=c(0, 62))
        axis(1, at=c(0,bp[seq(4,48,4)],62), labels=c('6am', '7am', '8am', '9am', '10am', '11am', 'Noon', '1pm', '2pm', '3pm', '4pm', '5pm', '6pm', '7pm'))
        axis(2, at=seq(0, ymax, 50), labels= seq(0, ymax, 5*10))
        par(new=T)
#        print(paste('Yymin', yymin, ' YYmax', yymax))
 
	if(f==3 & !is.null(Nymax)){ 
	        plot(x=bp+0, nv, col='blue', type='l', xaxt='n', xlim=c(0, 62),yaxt='n', ylab='', xlab='', ylim=c(yymin, yymax))
 	        axis(4)
	} else {
#        plot(x=bp+0, nv, col='blue', type='l', xaxt='n', xlim=c(0, 62),yaxt='n', ylab='', xlab='', ylim=c(1, yymax))
	        plot(x=bp+0, nv, col='blue', type='l', xaxt='n', xlim=c(0, 62),yaxt='n', ylab='', xlab='', ylim=c(yymin, yymax))
 	       #axis(4, at=seq(0,10,2), labels=seq(0,10,2))
		axis(4)
 
	}
        polygon(x=c(0, bp+0,max(bp)+0,0),y=c(0,nv,0,0), col=blu1)
        legend('topright',  c('Value of ON loans (in millions)', 'Number of ON loans'), col=c('red', 'blue'), lwd=2)
	dev.off()
}





