#!R
# Supporting Rmatlab connection functions 
#	openLocalRmatlabConnection is for serial jobs, opening matlab on local host at specified port
#	openMatlabConnection should be used for parallel variants - using bsub_RMPI+Matlab 
# 

openLocalRmatlabConnection <- function(port=9999){
        # Function opens a matlab session to run in the background and opens communication between the two 
        require(R.matlab)
        Matlab$startServer(port=port)
        try(matlab <- Matlab(port=port))
	if(inherits(matlab, 'try-error')){ 
			
		stop(paste('Matlab connection failed on port ', port, '.', 'Check that port is free, or modify port to connect toin calling script (e.g. if from RempelBootstrapTest.R change `portnum <- 9999` to a different free local port. Can use netstat -tnlp to check for which ports are in use / listening. If orphaned process, may end it via `kill -9 9999`', sep=' ')) 
	}	
        if (!open(matlab)) throw("Matlab server is not running: waited 30 seconds.")
        # test that connections are functioning
        x <- 1
        setVariable(matlab, B=x)
        evaluate(matlab, "C = B + 1")
        Cr <- getVariable(matlab, 'C')
        if(Cr$C==2){
                cat('I/O between R and Matlab functioning properly.')
		success <- 0
        } else {
                cat('I/O not functioning properly. Check configurations.')
		success <- 1
        }
	OUT <- list(success=success, matlab=matlab)
	return(OUT)
}

# Code originally written by Boyan, modified by Mark
openMatlabConnection <- function(jobname, rank=0){
                # rank = mpi.comm.rank() if in parallel
                out_fname <- sprintf("%s_output_slave.%03d.txt", jobname, rank)
		Remote <- FALSE 
                zz <- file( out_fname, open="wt")
                sink( zz, append=TRUE )
                sink( zz, type="message", append=TRUE )
                require(R.matlab);
		MatlabServerPort <- Sys.getenv("MATLABSERVER_PORT")
		print(paste('MatlabServerPort', MatlabServerPort))
                matlab.port <- as.integer(Sys.getenv("MATLABSERVER_PORT"))
#		print(matlab.port)
#		print(paste('Rank is', rank))
#		rank <- as.numeric(rank)
                cat(sprintf("rank %03d will use matlab port %5d\n", rank, matlab.port))
                matlab <- Matlab(port=matlab.port, remote=Remote)
#		matlab <- Matlab()
                open(matlab)
		print('Attempted to open Matlab')
                success <- isOpen(matlab)
                if(success) {
                        setOption(matlab, "readResult/interval", 15)  # in seconds
                        setOption(matlab, "readResult/maxTries", 120)  # 15 sec times 60 tries = 15 minutes
                        matlab.diary <- sprintf("logfiles/%s_output_matlab.%03d.txt", jobname, rank)
                        if (file.exists(matlab.diary)) file.remove(matlab.diary)
                        evaluate(matlab, paste("diary ", matlab.diary ))
                        evaluate(matlab, paste("rank = ", rank, ";"))
			evaluate(matlab, "targdir = [pwd, '/tmp'];")
			evaluate(matlab, "setenv('TEMP',targdir)" ) # set temporary file location
                 }
		     x <- 1
		     setVariable(matlab, B=x)
		     evaluate(matlab, "C = B + 1")
		     Cr <- NULL
		    Cr <- getVariable(matlab, 'C', remote=Remote)
		    if(is.null(CR)) stop("get variable failed")
	            if(Cr$C==2){
            		    cat('I/O between R and Matlab functioning properly.\n')
		            Tsuccess <- 0
	            } else {
		                cat('I/O not functioning properly. Check configurations.\n')
                		Tsuccess <- 1
	            }	

	    	    OUT <- list(success=success, matlab=matlab, Tsuccess=Tsuccess)
    	         return(OUT)

}
