function PotentialLoans = BaselineDuplicateTreatment(AllPotentialLoans, transactionId_column, ONrate_column, intrate_column, methodTreatDuplicates, duplicateIndicatorValue, duplicateTreatmentValue)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Supplemental function for BaselineFurfine.m and RempelRefinement.m (to be called via FurfineControlScript.m)
%
%	Selects amongst the set of payment pairs with reasonable inferred interest rates
%		Utilizes either a closest to target criterion, or nearest to median of non-unique rates
%		(Alternative method of duplicate treatment,kNN/Rempel variant is contained in a separate script)
%
%
%	Inputs: Potential loans (n x 22 matrix) - obtained via BaselineFurfine()
%
%	Output: Potential loans with column 22 indicating:
%			0 - if uniquely identified
%			2 - if the same sender payment is matched to multple return payments
%			3 - if the same received payment is matched to multiple send payments
%			4+ indicates both payments are involved in multiple  matches
%
% Written
%	Rempel 2013
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

    unique_transaction_ids = unique(AllPotentialLoans(:,transactionId_column));
    IdCounts = histc(AllPotentialLoans(:,transactionId_column), unique(AllPotentialLoans(:, transactionId_column)));
    IdsWithDuplicates = unique_transaction_ids(IdCounts > 1);
    NumDuplicateIds = size(IdsWithDuplicates,1); % number of Ids which are paired with multiple counterparty Ids
    
    if NumDuplicateIds > 0
        
        for j = 1:NumDuplicateIds
            IdJ = IdsWithDuplicates(j);
            duplistIndx = find(AllPotentialLoans(:, transactionId_column)==IdJ);
            AllPotentialLoans(duplistIndx,size(AllPotentialLoans,2)) = AllPotentialLoans(duplistIndx, size(AllPotentialLoans,2)) + duplicateIndicatorValue;
            
            DupPotentialLoans = AllPotentialLoans(duplistIndx,:);
            if strcmp(methodTreatDuplicates, 'Hendry')
                z_minrate = find(abs(DupPotentialLoans(:,intrate_column) - DupPotentialLoans(:,ONrate_column))==min(abs(DupPotentialLoans(:,intrate_column) - DupPotentialLoans(:,ONrate_column))));
                
                % taking the median repayment time (minimum index of
                % them, if multiple)
                minrateIndx = find(min(abs(DupPotentialLoans(z_minrate,12) -median(DupPotentialLoans(z_minrate,12))))==abs((DupPotentialLoans(z_minrate,12)-median(DupPotentialLoans(z_minrate,12)))),1);
            end
            if strcmp(methodTreatDuplicates, 'Armantier')
                medRate = median(DupPotentialLoans(:,intrate_column));
                tmpRate = DupPotentialLoans(:,intrate_column) + rand(size(DupPotentialLoans,1),1)/10000; % perturbing rates by a random insignificant amounts to avoid tie breakers
                minrateIndx = find(min(abs(tmpRate - medRate))==abs(tmpRate - medRate),1);
            end
            
            if ~strcmp(methodTreatDuplicates, 'Rempel')
                selectedFromDupsObs = duplistIndx(minrateIndx);
                AllPotentialLoans(selectedFromDupsObs, size(AllPotentialLoans,2)) = duplicateTreatmentValue + AllPotentialLoans(selectedFromDupsObs, size(AllPotentialLoans,2)) - duplicateIndicatorValue;   
            end
        end
    end
    PotentialLoans = AllPotentialLoans;
end



