function ElapsedTime = ConvertTimeToElapsed(time, varargin)
% ElapsedTime - number of seconds since midnight
%
% Inputs: 
%	ime in vector numeric (hhmmss) format - required
%       potential additional arguments: 
%                   benchmark time (same format) - benchmark is the
%                   starting time (time elapsed from point. e.g benchmark =
%                   midnight implies, 6apm is 6 hours after the benchmark)
%                   date of time (yyyymmdd format)
%                   date of benchmark
%
%
% Output: numeric vector of elapsed time in seconds since midnight in a day
% Rempel, 2013

    if iscell(time)
        timec = cellfun(@(x) num2str(x), time, 'UniformOutput', false);
    else
        timec = arrayfun(@(x) num2str(x), time, 'UniformOutput', false);
    end
    %timec = cellfun(@(x) num2str(x), time, 'UniformOutput', false);
    % for timec with less than 6 characters, pad with 0's

    timec = cellfun(@(x) [repmat('0', length(find(length(x)<6)), 6- size(x,2)) x], timec, 'UniformOutput', false);
    nargs = length(varargin);
    if nargs == 0 
        
        datec = '20120101';
        benchmark = '20120101.000000';
        
    else
        benchmarkdate = '20120101';
        benchmarktime = varargin{1};
        if nargs > 1
            datec = varargin{2};
            
            if nargs > 2
                benchmarkdate =varargin{3};
            end
        end
        
    end
    if size(datec,1) > 1
          
            datec = arrayfun(@(x) num2str(x), datec, 'UniformOutput', false);
            benchmarkdate = arrayfun(@(x) num2str(x) , benchmarkdate, 'UniformOutput', false);
            benchmarktime = arrayfun(@(x) num2str(x) , benchmarktime, 'UniformOutput', false);
            ElapsedTime = cellfun(@(w,x,y,z) ConvertTimeToElapsedScalars(w,y,x,z), timec, datec, benchmarktime, benchmarkdate, 'UniformOutput', false);
          
        
    else
        benchmark = datevec(datenum(benchmark, 'yyyymmdd.HHMMSS')); 
        datetimec = cellfun(@(x) datevec(datenum([datec '.' x], 'yyyymmdd.HHMMSS')), timec, 'UniformOutput', false);
        ElapsedTime = cellfun(@(x) etime(x, benchmark), datetimec, 'UniformOutput', false);
    end
    ElapsedTime = cell2mat(ElapsedTime);
end

