function [InterestIsRound, varargout] = InterestIsRounded(principal,repayment,intrate, varargin)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Interest is rounded criterion - to be called from GeneratingKNNcovariates.m
%	For interest rate of x.abcde%, check that c is 0 or 5 (after rounding to nearest c)
%			that is that the implied interest rate is rounded to the nearest basis/half-basis point
%				account for rounding to the nearest cent?
%
%	y = return payment, x = principal payment
%	ir = (y/x - 1)/365*100 yields interest rate in % points
%	
%	if 		ir = z.abcde s.t. z.abc'is 0 or 5 (c' is z.abcde rounded to the nearest c)
%	AND
%			y = rounded to nearest cent(z.abc' * x)
%	then	
%		interest rate is considered rounded
%	else
%		it is not rounded
%	end
%
%
%	Output:
%		Indicator of interest rate is round to nearest bp or half bp, and rounded rate * principal matches repayment
%		Less restrictive indicator, simply of interest rate tenth of bp is 0 or 5
%		Value of tenth of bp digit (post rounding up to the tenth of bp)
% Rempel, 2014
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

	% intrate is in percentage points
	ind = zeros(length(principal),1);
	ind2 = zeros(length(principal),1);
	%length(principal)
	nearestBP = round(intrate*1000)/10; % interest rate rounded to nearest tenth of a bp
	nearestBPdigit = nearestBP - floor(nearestBP); % tenth of bp is now the first decimal place
	% will check if the tenth digit is 0 or 5
	nearestBPdigit= round(nearestBPdigit*10);
	IsHalfBP= find(nearestBPdigit==5 | nearestBPdigit==0);
	

 	if length(varargin)>0
		num_days = varargin{1};
	else
		num_days = ones(length(principal),1);
	end
	onr = (repayment./principal - 1).*(365./num_days);
	onr_r = round(onr*10^5)/10^5; % round to nearest basis point
	repayment_round = (onr_r./(365./num_days) + 1).*principal;
	ReturnDiff = repayment*100 - round(repayment_round*100);
	IsExact = find(ReturnDiff==0);
	ind(intersect(IsHalfBP, IsExact)) = 1;
	ind2(IsHalfBP) = 1;
	InterestIsRound = ind;
	varargout{1} = ind2;
	varargout{2} = nearestBPdigit;
	varargout{3} = ReturnDiff;
end

	


