function Predictions = RempelKNNpredict(Xpred, varargin)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Supplemental function for k-NN refinement
%	Called from FurfineControlScript.m
%
%	Compares the potential loan covariates against the trained data (unique/faux loans) using K-NN
%			See Trained_Data / RempelKNNtraining for details 
%
%	if more than a training model is supplied, presumed that Discriminant adaptive nearest neighbours to be performed
%		this is a more localized variant of k-NN, which is much more computationally demanding
%			this method not discussed in Rempel (2014) 
%
%	Inputs:
%		Xpred = Nonuniquely identified loan covariates
%		if length(varargin) = 1
%			Trained Data
%		else
%			UniqueLoans/Synthetic payment covariates and indicators in matrix forms
%				(Xtrain, Ytrain) form
%		end
%
%	Output:
%		Vector of predictions (1/0) corresponding to the order of payments in Xpred
%
% Rempel, 2013
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
nargs = length(varargin);
if nargs == 1
    % then T
    disp('Using trained model')
    TrainingModel = varargin{1};
    
    [Ypred, ~, ~] = predict(TrainingModel, Xpred); % second to args are score/cost
                        
else
   error('KNNpredict:argsIssue', 'Additional arguments unsupported for RempelKNNpredict')
end

Predictions = Ypred;


    
