%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Refinemennt as given in Rempel (2016)
%
%	Must be used in conjunction with FurfineControlScript.m
%
%	Key Input: FurfineData, column 22 = 0 indicates uniquely identified loans
%	
%	Output: PredData gives the FurfineData  with the uniquely identified loans and
%		the loans which more closely resemble the unique loans vs randomly paired payments
%		between the same two participants 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

if ModelTrainingData == 1
	%% Making Classification model

	% 1) generate training data
       TrueLoans = FurfineData(FurfineData(:,22)==0,:); % Hypothesized as true, given they were identified uniquely
       TrainedDataI = Training_Data(TrueLoans, lvtsdatafile, ControlSyntheticForParticipants); 
       TrainedXs = GeneratingKNNcovariates(TrainedDataI);
       TrainedYs = TrainedDataI(:,22); % 1's indicate FauxLoans, 0's indicate Uniquely Id'd Loans
       TrainedYs(TrainedYs > 0) = 1; % 

	% 2) Making Classification model - user specified k or via cross-validation
       if ~isempty(choiceK)
		% user specified K for k-nearest neighbours classification 
	       modelI = RempelKNNtraining(TrainedXs, TrainedDataI(:,22), choiceK);
       else

		% automatically determine the optimal `K' in K nearest neighbours
	    if ~isempty(klossWeighted) && ~isempty(maxk)
		      % using kloss binomial deviance weighting (=1) or unweighted (0)
			% and range of k under consideration from 1 - maxK (by 4)
			modelI = RempelKNNtraining(TrainedXs, TrainedDataI(:,22), klossWeighted, maxk, Parallel);

	    else
			% using defaults (checking grid between K = 1, and 51)
		       modelI = RempelKNNtraining(TrainedXs, TrainedDataI(:,22));
	    end
      end

else
	% Presumed model has already been generated
	% and is loaded
	disp('Training Model (modelI) taken as given.')
end

%% Predicting the non-uniquely identified loans using modelI

PredictorDataFurf = FurfineData(FurfineData(:,22)>0, :);
PredictorData = GeneratingKNNcovariates(PredictorDataFurf);
Predictions = RempelKNNpredict(PredictorData, modelI);

%% Post Prediction Clean Up - applying secondary duplicate criterion
PredData = PredictorDataFurf(Predictions==0,:); % kNN predicted overnight loans
PredData(:,22) = 0; % resetting column 22 to 0's as DuplicateCriterion will modify it to other integers if not unique 

PredData = BaselineDuplicateTreatment(PredData, senderTransactionIdColumn, ONrate_column, intrate_column, methodDup2, SenderDuplicateIndicatorValue, SenderSelectedDuplicateValue);

loansNonUniqueAndNotSelected = PredData(PredData(:,22) > SenderSelectedDuplicateValue,:);
fprintf('About to get Pred Data %d', SenderSelectedDuplicateValue)
PredData = PredData(PredData(:,22) <= SenderSelectedDuplicateValue,:);

% Receiver payment duplication (same as sender)

PredData = BaselineDuplicateTreatment(PredData, receiverTransactionIdColumn, ONrate_column, intrate_column, methodDup2, ReceiverDuplicateIndicatorValue, ReceiverSelectedDuplicateValue);
PredData = PredData(PredData(:,size(PredData,2))<= SenderSelectedDuplicateValue + ReceiverSelectedDuplicateValue,:); % <= 5 with default sender/receiverSelected values
% Each PredData row now has a unique sender transaction id and receiver transaction id

