#! R

# Preparing the LVTS data for running Furfine-based algorithms on sampling

# Paritioning the original LVTS data into pre-crisis, crisis, effective lower bound and post-effective lower bound periods


# Defining subsample cut-off dates
BNP <- as.Date('20070809', format='%Y%m%d')
ELB_s <- as.Date('20090421', format='%Y%m%d')
ELB_e <- as.Date('20100531', format='%Y%m%d')

load('FurfineInputFile_2005-2011_rlvts.RData')
	# rlvts is just the lvts data with date in the date format rather than numeric
		# note, it excludes all payments involving the Bank of Canada (sender/receiver code 2)
# 1) ensuring no payments prior to 6 a.m. and after 6:30 p.m.

rlvts <- rlvts[-which(rlvts$time < 60000 | rlvts$time > 183000),]

lvts <- rlvts[which(rlvts$date < BNP),]
save(lvts, file='PrecrisisLVTS.RData')

lvts <- rlvts[which(rlvts$date >= BNP & rlvts$date < ELB_s),]
save(lvts, file='CrisisLVTS.RData')

lvts <- rlvts[which(rlvts$date >= ELB_s & rlvts$date <= ELB_e),]
save(lvts, file='ELB_LVTS.RData')

lvts <- rlvts[which(rlvts$date > ELB_e),]
save(lvts, file='PostELB_LVTS.RData')

rm(lvts)
rm(rlvts)



# 2) remove holidays and the days immediately preceding the holiday (e.g. remove Friday preceding holiday Monday)


MartinLuther <- c(20050117,20060116, 20070115, 20080121, 20090119, 20100118, 20110117)
Memorial <- c(20050530, 20060529, 20070528, 20080526, 20090525, 20100531, 20110530)
Independence <- c(20050704, 20060704, 20070704, 20080704, 20090703, 20100705, 20110704)
USthanksgiving <- c(20051124, 20061123, 20071122, 20081127, 20091126, 20101125, 20111124)
USholidays <- sort(c(MartinLuther, Memorial, Independence, USthanksgiving))

# construct Canada Holiday indicator (pervasive Province instituted holidays, federal still has to work)

Aug1stCivic <- c(20050801, 20060807, 20070806,20080804, 20090803, 20100802, 20110801)
FamilyDay <- c(20050221, 20060220, 20070219, 20080218, 20090216, 20100215, 20110221)
Canada_NonFedHolidays <- sort(c(Aug1stCivic, FamilyDay))

Holidays <- c(Canada_NonFedHolidays, USholidays)
Holidays <- as.Date(as.character(Holidays), format='%Y%m%d')
rlvts <- rlvts[which(rlvts$date %in% Holidays),]


lvts <- rlvts[rlvts$date < BNP),]
save(lvts, file='PrecrisisLVTS.RData')

lvts <- rlvts[which(rlvts$date >= BNP & rlvts$date < ELB_s),]
save(lvts, file='CrisisLVTS.RData')

lvts <- rlvts[which(rlvts$date >= ELB_s & rlvts$date <= ELB_e),]
save(lvts, file='ELB_LVTS.RData')

lvts <- rlvts[which(rlvts$date > ELB_e),]
save(lvts, file='PostELB_LVTS.RData')







