#! R

##################################################################################################
# Script that applies the Furfine algorithm and/or variants to the re-paired days
#
# Inputs (args):
#	BootstrapSample RData file
#	
#
#
#
###################################################################################################


# RempelBootTest.R --args 'BootstrapSample.RData', 'OutputName.RData', 'Rempel', Parallel, nMatSlaves, SyntheticTraining, ReturnLeg, Control, useDANN, kloss, choiceK, Xpaythreshold, methodDup2, min_sentime, min_loansize, bpsl, bpsh, refinement, training suffix

## Initialization
print('starting Rempel BootTest')

args <- commandArgs(trailingOnly = TRUE)
inputfilename <- args[1]
outputfilename <- args[2]
sprintf('for output %s', outputfilename)
methodTreatDuplicates <- args[3]
Parallel <- as.numeric(args[4])
nMatSlaves <- as.numeric(args[5])
min_sentime <- as.numeric(args[6])
min_loansize <- as.numeric(args[7])
bpsl <- as.numeric(args[8])
bpsh <- as.numeric(args[9])
refinement <- args[10]
print(refinement)
print(min_sentime)
print(min_loansize)
if(length(args) > 10){
# then it is the additional options for Rempel Criterion
	SyntheticTraining <- as.numeric(args[11])
	ControlSyntheticForParticipants <- as.numeric(args[12])
        useDANN <- as.numeric(args[13])
        klossWeighted <- as.numeric(args[14])
        choiceK <- as.numeric(args[15])
        Xpaythreshold <- as.numeric(args[16])
#	ModelTraining = args[13] # 
	methodDup2 <- args[17]
	training_suffix_str <- args[19]
	training_prefix_str <- args[18]	
	ModelTraining <- as.numeric(args[20])
#	lvtsdatafile <- args[20]
	lvtsdatafile <- "Inputs/LVTS_2005-2011ordered.mat"
#	training_suffix_str
#	print(paste('ModelTrainingData ' , ModelTrainingData))

}
jobname <- sub('.RData', '', outputfilename)
mpi.bcast.Robj2slave(jobname)
#ModelTrainingData <- 0
#mpi.bcast.Robj2slave(ModelTrainingData)
Simulation <- 1
mpi.bcast.Robj2slave(Simulation)
#if(refinement=='EndOfDay'){
	endofdaythreshold <- 160000
#}
mpi.bcast.Robj2slave(endofdaythreshold)

load(inputfilename)
###############Initialize slave ###################################

success <- mpi.remote.exec(try({
    out_fname <- sprintf("%s_output_slave.%03d.txt", jobname, mpi.comm.rank())
    zz <- file( out_fname, open="wt")
    sink( zz, append=TRUE )
    sink( zz, type="message", append=TRUE )

    require(R.matlab);
    matlab.port <- as.integer(Sys.getenv("MATLABSERVER_PORT"))
    cat(sprintf("rank %03d will use matlab port %5d\n", mpi.comm.rank(), matlab.port))
    matlab <- Matlab(port=matlab.port)
    open(matlab)
    success <- isOpen(matlab)
    if(success) {
        setOption(matlab, "readResult/interval", 15)  # in seconds
        setOption(matlab, "readResult/maxTries", 120)  # 15 sec times 60 tries = 15 minutes
        matlab.diary <- sprintf("%s_output_matlab.%03d.txt", jobname, mpi.comm.rank())
        if (file.exists(matlab.diary)) file.remove(matlab.diary)
        evaluate(matlab, paste("diary ", matlab.diary ))
        evaluate(matlab, paste("rank = ", mpi.comm.rank(), ";"))
    }
    success
}), ret=TRUE)

if (any(sapply(success, inherits, "try-error")) || !all(as.logical(success))) {
    mpi.bcast.cmd( close(matlab) )
    mpi.close.Rslaves()
    mpi.quit()
}

###########################################################################





mpi.bcast.Robj2slave(nMatSlaves)
FurfineScript <- 'FurfineControlScript'
mpi.bcast.Robj2slave(FurfineScript)


# Furfine Script options
Nchunks = 1;

# sending all the parameters to the slaves:

if(length(args) > 10){
	useReturnLegCharacteristics <- 1
	mpi.bcast.Robj2slave(SyntheticTraining)
	mpi.bcast.Robj2slave(useReturnLegCharacteristics)
        mpi.bcast.Robj2slave(ControlSyntheticForParticipants)
	mpi.bcast.Robj2slave(training_suffix_str)
	mpi.bcast.Robj2slave(training_prefix_str)
	mpi.bcast.Robj2slave(useDANN)
	mpi.bcast.Robj2slave(klossWeighted)
	mpi.bcast.Robj2slave(outputfilename)
	mpi.bcast.Robj2slave(Xpaythreshold)
	mpi.bcast.Robj2slave(ModelTraining)
	mpi.bcast.Robj2slave(choiceK)
	mpi.bcast.Robj2slave(methodDup2)
	mpi.bcast.Robj2slave(lvtsdatafile)
#        mpi.bcast.Robj2slave(training_suffix_str)

}

mpi.bcast.Robj2slave(nMatSlaves)
mpi.bcast.Robj2slave(Parallel)
mpi.bcast.Robj2slave(Nchunks)
mpi.bcast.Robj2slave(methodTreatDuplicates)
mpi.bcast.Robj2slave(min_sentime)
mpi.bcast.Robj2slave(min_loansize)
mpi.bcast.Robj2slave(bpsl)
mpi.bcast.Robj2slave(bpsh)
mpi.bcast.Robj2slave(refinement)
mpi.bcast.Robj2slave(outputfilename)




# Setting parameters in the matlab worker workspaces:

mpi.bcast.cmd(print('Passing parameters to mat workers\n'))

#mpi.bcast.cmd(matlab.send.cmd.1(disp('Getting parameters from R:'))

mpi.bcast.cmd(evaluate(matlab, "fprintf('[%d] Getting parameters', rank);"))
mpi.bcast.cmd(setVariable(matlab, Nchunks=Nchunks,
        Parallel=Parallel, nMatSlaves=nMatSlaves, min_sentime = min_sentime,
        min_loansize=min_loansize, methodTreatDuplicates=methodTreatDuplicates,
        refinement=refinement, bpsh=bpsh, bpsl=bpsl, Simulation=Simulation,
        output_string = outputfilename
))
if(length(args) > 10){
    mpi.bcast.cmd(setVariable(matlab, Xpaythreshold=Xpaythreshold,
            choiceK=choiceK,
            ControlSyntheticForParticipants=ControlSyntheticForParticipants,
            useReturnLegCharacteristics=useReturnLegCharacteristics,
            useDANN=useDANN,
            klossWeighted=klossWeighted,
            ModelTrainingData=ModelTraining,
            methodDup2=methodDup2
    ))
}

if(refinement=='EndOfDay'){
	mpi.bcast.cmd(setVariable(matlab,endofdaythreshold=endofdaythreshold))
}

## Starting

        # adding additional matlab libraries
mpi.bcast.cmd(evaluate(matlab, "addpath('/scratch/remm/LVTS/ImprovingOvernightLoanIdentification/KNN'); \n")) # addpath('/home/mfa/tjac/Payments/RempelFiles/KNN')
mpi.bcast.cmd(print("Main Function About To Start"))


# tags sent by the master to the slaves
master.tag.work <- 1
master.tag.done <- 2
# tags sent by the slaves to the master
slave.tag.ready <- 101
slave.tag.finished <- 102
slave.tag.quit <- 103

mpi.bcast.cmd({ master.tag.work <- 1; master.tag.done <- 2;
                slave.tag.ready <- 101; slave.tag.finished <- 102; slave.tag.quit <- 103})

slavefunction <- function(){
        #starttime = proc.time()[3]
        junk <- 0
        done <- 0
        iter.counter = 0
        # on default communicator = 1
        # communicator = 1
        print("In Slave Function")
        flush.console()
        while(done !=1){
                starttime = proc.time()[3]
                iter.count = iter.counter + 1
                master.node <- 0
       print(sprintf("sending tag %d", slave.tag.ready))
                mpi.send.Robj(obj=junk,dest=master.node, tag=slave.tag.ready) # , comm=communicator)
                task <- mpi.recv.Robj(source=mpi.any.source(), tag=mpi.any.tag()) # , comm=communicator)
                task.info <- mpi.get.sourcetag()
                tag <- task.info[2]
       print(sprintf("received tag %d", tag))
                if(tag==master.tag.work){
                        # perform task and create results
                        pt.start <- proc.time()
                        b <- task$bootNumber
        print(sprintf("sample number %d received", b))
                        my.sample <- task$bootsample
                        class(my.sample)
                        dim(my.sample)
			setVariable(matlab, b=b)
			evaluate(matlab, "fprintf('b is %d', b)\n")
			setVariable(matlab, lvts=as.matrix(my.sample))
                       	evaluate(matlab, "size(lvts)") 
			print('Sending sample')
#			evaluate(matlab, "fprintf('refinement is %s\n', refinement)\n")
			
			if(methodTreatDuplicates=='Rempel'){
				print('Rempel duplicate method used')
				if(methodDup2=='Hendry'){
					print('loading training data in matlab')
					#evaluate(matlab, "CMDstringBase = strtrim('", training_prefix_str, "'
					#matlab.send.cmd.1("CMDstringBase = strtrim('", training_prefix_str, "');\n", file=matlab)
					CMDstringR <- paste('TrainingData/', training_prefix_str, b, training_suffix_str, sep='')
					setVariable(matlab, CMDstring = CMDstringR)
#					setVariable(matlab,training_prefix_str = training_prefix_str)
#					setVariable(matlab,training_suffix_str = training_suffix_str)
#					evaluate(matlab, "baseCMDstring = ['TrainingData/' training_prefix_str]")
					#matlab.send.cmd.1("disp('Loading Training data')")
#					evaluate(matlab, "CMDstring = [baseCMDstring num2str(b)]")
#					mat.cmdstring <- getVariable(matlab, CMDstring)
#                                        cat('Matlab CMD string', mat.cmdstring)
#					evaluate(matlab, "CMDstringSuffix = strtrim(training_suffix_str)")
#					evaluate(matlab, "CMDstring = strtrim([CMDstring CMDstringSuffix])")
#					mat.cmdstring <- getVariable(matlab, CMDstring)
#                                        cat('Matlab CMD string final', mat.cmdstring)

					evaluate(matlab, "load(CMDstring)")
					print('command to matlab to load has been given, waiting for task completion..')
					evaluate(matlab, 'class(modelI)') 					
					print('model data loaded')
				} else {
					#cat(paste("load('TrainingData/TrainedLVTS_", b, "_Armantier.mat')", sep=''),"\n", file=matlab)
				}
			}
			getVariable(matlab, 'refinement')$refinement -> refinement.m
			print(paste('refinement in matlab is', refinement.m))
			evaluate(matlab, FurfineScript)
			
			if(!refinement=='None'){
				#percentremoved <- getVariable(matlab, 'percentremoved')$percentremoved
				#print(sprintf('percent removed %f from refinement %s', refinement, percentremoved))
				print('here')
			}

                        # applying Furfine Algorithm to the lvts data
                         # putting in the full non-parallel script. 
			Fresults <- getVariable(matlab, 'Loans')$Loans
                        print('Getting Loans data from Matlab into R')
			print(class(Fresults))
			print(head(Fresults))
                        evaluate(matlab, "disp('Waiting for R instruction')")
                        comp.time <- proc.time() - pt.start
                        if(Fresults[1,1]==0){
                                # no loans identified!
                                print('No loans identified in this sample')

                                results <- list(bootNumber=b, num_loans=0, matches = NULL, comp.time=comp.time, num_nonunqiue = NULL, num_obs=nrow(my.sample))

                        } else {
                                print('Loans identified in this sample')
                                num_nonunique = length(Fresults[,22]>0)
                                numloans = nrow(Fresults)
                                results <- list(bootNumber=b, num_loans=numloans, matches=Fresults, comp.time = comp.time, num_nonunique = num_nonunique, num_obs=nrow(my.sample))
                        }
                        

                         # Rslave receiving data from the pipe
                        #Fresults <- matlab.recv.matrix(data.fifo.name)
        print(sprintf("Results from matlab %d", Fresults[1,1]))
                        # clearing un-needed data
                        rm(my.sample)

        print('Sending Results')
                        mpi.send.Robj(obj=results, dest=master.node, tag=slave.tag.finished)
                } else if (tag==master.tag.done){
                        # Master has no more tasks. Exit
                        done <- 1
                        print("Master has said there are no more jobs, exiting")
                } else {
                        # unaccounted for message..leave this code section blank to ignore
                        #print('Unanticipated message')


                }
                flush.console()
                if(proc.time()[3] > starttime*100){
                        print(paste("Run-time Hangup on", mpi.comm.rank()))
                        #mpi.send.Robj(junk,master.node, tag=3)
                        break
                }

        }
        # Tell Master that this slave is quiting. 
        mpi.send.Robj(junk, master.node, tag=slave.tag.quit)
}

mpi.bcast.Robj2slave(slavefunction)
mpi.bcast.cmd(slavefunction())







# Master Function




numboots <- length(BootSample)
tasks <- vector('list', length=numboots)
tasks <- lapply(1:numboots, function(x) list(bootNumber=x, bootsample=BootSample[[x]]$bootsample))
#print(tasks[[numboots]])
Results <- vector('list', length=numboots)
junk <- 0
closed_slaves <- 0
n_slaves <- mpi.comm.size() - 1
while(closed_slaves < n_slaves){
        # receive any message from any slave
        proc.time()[3] -> master.initial.time
        slave.msg <- mpi.recv.Robj(mpi.any.source(), mpi.any.tag())
        slave.msg.info <- mpi.get.sourcetag()
        slave_id <- slave.msg.info[1]
        slave_tag <- slave.msg.info[2]
        print(paste('slave id', slave_id, ' slave_tag ', slave_tag))
        if (slave_tag==slave.tag.ready) {
                if(length(tasks) > 0){
                        this.task <- tasks[[1]]
                        tasks[1] <- NULL
                        print(paste('Number of tasks left -', length(tasks)))
                        #this.task$bootSample <- BootSample[[this.task$bootNumber]]
                        mpi.send.Robj(this.task, slave_id, tag=master.tag.work);
                        #break
                } else mpi.send.Robj(obj=NULL, dest=slave_id, tag=master.tag.done)
        } else if (slave_tag==slave.tag.finished) {
                # then we're receiving results
                print(paste('Received Results', 'Subbing them into Results', sep='\n'))
                Results[[slave.msg$bootNumber]] <- slave.msg
                #mpi.bcast.cmd(BootSample[[slave.msg$bootNumber]] <- NULL)

                # removing this sample from all of the slaves to save space?
                save(Results, file='TempResultsApr12TestRmpiPull.RData')

        } else if (slave_tag==slave.tag.quit) {
                # A slave has shut down
                print('Slave has closed down')
                closed_slaves <- closed_slaves + 1
                if (n_slaves - closed_slaves==1){
                        proc.time()[3] -> master.last.slave.time
                }
        }
        flush.console()
        if(n_slaves - closed_slaves==1){
                if(master.last.slave.time + 120 < proc.time()[3]){
                        print('Master has hung-up on the last task')
                        break
                }
        }
        if(master.initial.time*100 < proc.time()[3]){
               print(paste("Master is presumed to have hung-up"))
               break
        }
}

sprintf("Job has run it's course. Saving Results in %s", outputfilename)
save(Results, file=paste('Results/', outputfilename, sep=''))

print('Saved')

cat("closing matlabs\n")
mpi.bcast.cmd(close(matlab))
mpi.close.Rslaves()
mpi.quit()



