#!R
RempelModelFunction <- function(task, furfdat){
	# perform task and create results
	# task: for a given b,  
	#	    1) get the ON rate for the LVTS samples
	#           2) draw from the first round Rempel furfine loans with that same ONrate
	#           3) restrict these furfine to those uniquely identified
	#           4) Run training_data, GenKNN, and RempelKNNtraining
	#           5) save output (modelI) in a .mat

	pt.start <- proc.time()
	b <- task$bootNumber
	print(sprintf("sample number %d received", b))
	my.sample <- task$bootsample
	class(my.sample)
	dim(my.sample)
	evaluate(matlab, "fprintf('[%d] Preparing to receive my sample matrix\\n', rank);")
	setVariable(matlab, b=b)

	# 1) get the ONrate
	bONrate = unique(my.sample[,7])
	cat("bONrate = ", bONrate, "\n")

	# get number of days in sample
	numdays <- length(unique(my.sample[,1]))

	# 2) sample from Furfine data
		# a) sample unique dates from the Furfine data (without replacemnt)
	udates <- as.Date(as.character(unique(furfdat[which(furfdat[,7] %in% bONrate),1])), format='%Y%m%d')
	if(length(udates) < numdays){
		# if there are insufficient days in the original sample for the number of days resampled,
		# sample with replacement
		furfdates <- gsub('-', '', as.character(sample(udates,  numdays, replace=T)))
	} else {
		furfdates <- gsub('-', '', as.character(sample(udates,  numdays, replace=F)))
	}
	cat("numdays = ", numdays, "\nmy.sample:\n")
	print(unique(my.sample[,1]))
	cat("udates:\n")
	if( length(udates) < 40 ) {
		print(udates)
	} else {
		print(summary(udates))
	}
	# b) for each date selected, take all of the unique Rempel data therefrom
	furfsample <- furfdat[which(furfdat[,1] %in% furfdates),]

	# 3) restrict to only unique loans
	furfsample <- furfsample[which(furfsample[,22]==0),]

	# 4) run training_data, GenKNN and RempelKNNtraining

	# a) send the furfine data to matlab
	setVariable(matlab, FurfineData=as.matrix(furfsample))

	# b) send the lvts sample
	evaluate(matlab, "size(FurfineData)")
	setVariable(matlab, lvts=as.matrix(my.sample))
	evaluate(matlab, "size(lvts)")
	evaluate(matlab, FurfineScript)

	# 5) save output (modelI) in .mat file
	evaluate(matlab, "fprintf('[%d] Command after algorithm\\n', rank);")
	evaluate(matlab, paste("save('", outputfilename, "_", b, "Sample_", subsample_number, ".mat', 'modelI', '-v7.3');", sep=''))
	print('removing sample')
	# clearing un-needed data
	rm(my.sample)
	results <- list(bootNumber = b, numloans = 0, blah = NULL)
	print('Sending Results')
	return(results)
}


slavefunction <- function(){
        junk <- 0
        done <- 0
        iter.counter = 0
	# on default communicator = 1
	# communicator = 1
        print("In Slave Function")
        flush.console()
        while(done !=1){
                starttime = proc.time()[3]
                iter.count = iter.counter + 1
                master.node <- 0
	        print(sprintf("sending tag %d", slave.tag.ready))
                mpi.send.Robj(obj=junk,dest=master.node, tag=slave.tag.ready) # , comm=communicator)
                task <- mpi.recv.Robj(source=mpi.any.source(), tag=mpi.any.tag()) # , comm=communicator)
                task.info <- mpi.get.sourcetag()
                tag <- task.info[2]
	        print(sprintf("received tag %d", tag))
                if(tag==master.tag.work){
			results <- RempelModelFunction(task, furfdat)
		        print('Sending Results')
                        mpi.send.Robj(obj=results, dest=master.node, tag=slave.tag.finished)
  		} else if (tag==master.tag.done){
		# Master has no more tasks. Exit
                        done <- 1
                        print("Master has said there are no more jobs, exiting")
		} else {
	                flush.console()
        	        if(proc.time()[3] > starttime*100){
                	        print(paste("Run-time Hangup on", mpi.comm.rank()))
#mpi.send.Robj(junk,master.node, tag=3)
                        	break
			}
		}
	}	
	# Tell master that this slave is quitting
	mpi.send.Robj(junk, master.node, tag=slave.tag.quit)
}
