#! R

# Simulating payments data

generate_dailynonloan_paymentflows <- function(num_participants, num_nonloanpayments, payment_mean.per.participant, payment_sd.per.participant, ONrate, FXrate,  day){

        nonloan_payments.i <- vector('list', num_participants)
        time.payments.i <- vector('list', num_participants)
        sender.i <- vector('list', num_participants)
        receiver.i <- vector('list', num_participants)
        for(i in 1:num_participants){

                nonloan_payments.i[[i]] <- abs(round(rnorm(num_nonloanpayments, mean=payment_mean.per.participant*i, sd=payment_sd.per.participant*(10^(i/2) )), digits=2))
                time.payments.i[[i]] <- round(runif(num_nonloanpayments, 60000, 180000), digits=0)
                sender.i[[i]] <- rep(i, num_nonloanpayments)
                counterparty <- 1:4
                counterparty[-which(counterparty==i)] -> counterparty
                receiver.i[[i]] <- sample(counterparty, num_nonloanpayments, replace=T)
        }
        output <- cbind(rep(day, num_nonloanpayments*num_participants),  do.call(c, time.payments.i), do.call(c, nonloan_payments.i), do.call(c, sender.i), do.call(c, receiver.i), rep(FXrate, num_FX*num_participants), rep(ONrate, num_FX*num_participants), rep(0,num_FX*num_participants),rep(0,num_FX*num_participants))
        return(output)
}

generate_FXpayments  <- function(num_participants, num_FX, ONrate, FXrate, day){
        # randomly determine the FXrate per FX transaction


        FX_payments.i <- vector('list', num_participants)
	FX_const_payments.i <- vector('list', num_participants)
        time.const.payments.i <- vector('list', num_participants)
        time.payments.i <- vector('list', num_participants)
        sender.i <- vector('list', num_participants)
        receiver.i <- vector('list', num_participants)
        for(i in 1:num_participants){
                FXrates.d <- abs(round(rnorm(num_FX, mean=FXrate, sd=0.001)  , digits=4)) # FX quoted by pips
                FXsize.d <- seq(10^6, 5*10^6, by=10^6)
                FX_payments.i[[i]] <- FXrates.d*FXsize.d
		FX_const_payments.i[[i]] <- sample(FXsize.d, num_FX, replace=T)
                time.payments.i[[i]] <- round(runif(num_FX, 60000, 130000), digits=0)
                time.const.payments.i[[i]] <- round(runif(num_FX, 60000, 150000), digits=0)
                sender.i[[i]] <- rep(i, num_FX)
                counterparty <- 1:4
                counterparty[-which(counterparty==i)] -> counterparty
                receiver.i[[i]] <- sample(counterparty, num_FX, replace=T)
        }
        output <- cbind(rep(day, num_FX*num_participants),  do.call(c, time.payments.i), do.call(c, FX_payments.i), do.call(c, sender.i), do.call(c, receiver.i), rep(FXrate, num_FX*num_participants), rep(ONrate, num_FX*num_participants), rep(1,num_FX*num_participants),rep(0,num_FX*num_participants))

	const.output <- cbind(rep(day, num_FX*num_participants),  do.call(c, time.const.payments.i), do.call(c, FX_const_payments.i), do.call(c, sender.i), do.call(c, receiver.i), rep(FXrate, num_FX*num_participants), rep(ONrate, num_FX*num_participants), rep(2,num_FX*num_participants),rep(0,num_FX*num_participants))

	Output <- rbind(output, const.output)

        return(Output)
}


# generate overnight loans

generate_overnightloans <-  function(num_participants, num_ONloans, ONrate, FXrate_s, FXrate_r, meanONloan.size, sdONloan.size, day_s, day_r){
	ond <- 365 
        ONloans.i <- vector('list', num_participants)
        ONrepayments.i <- vector('list', num_participants)
        time.payments.i <- vector('list', num_participants)
        time.repayments.i <- vector('list', num_participants)
        sender.i <- vector('list', num_participants)
        receiver.i <- vector('list', num_participants)
	t <- as.numeric(as.Date(as.character(day_r), format='%Y%m%d') - as.Date(as.character(day_s), format='%Y%m%d') )
        for(i in 1:num_participants){
#                ONrates.d <- round(rnorm(num_ONloans, mean=ONrate, sd=0.05)  , digits=2) # ON rates quoted to nearest basis points
		ONrates.d <- rep(ONrate, num_ONloans)
		principal <-  abs(round(rnorm(num_ONloans, mean = meanONloan.size, sd=sdONloan.size)/10^6, digits=0))*(10^6) # million dollar increments, loan principal transfer
		repayment <- round(principal*(1 + ONrates.d/100*t/ond), digits=2)
#		stopifnot(length(which(repayment==0))==0)
                ONloans.i[[i]] <- principal
                ONrepayments.i[[i]] <- repayment # principal size x (1 + Overnight Percent Rate per Annum/100 *(term of loan)/num days in annum
                time.payments.i[[i]] <- round(runif(num_ONloans, 140000, 183000), digits=0)
                time.repayments.i[[i]] <- round(runif(num_ONloans, 60000, 140000), digits=0)
                sender.i[[i]] <- rep(i, num_ONloans)
                counterparty <- 1:4
                counterparty[-which(counterparty==i)] -> counterparty
                receiver.i[[i]] <- sample(counterparty, num_ONloans, replace=T)
#		chk <- c(day_s, time.payments.i[[i]], principal, sender
        }
        output.principal <- cbind(rep(day_s, num_ONloans*num_participants),  do.call(c, time.payments.i), do.call(c, ONloans.i), do.call(c, sender.i), do.call(c, receiver.i),  rep(FXrate_s, num_ONloans*num_participants), rep(ONrate, num_ONloans*num_participants), rep(0, num_ONloans*num_participants) ,rep(1, num_ONloans*num_participants))

        output.repayment <- cbind(rep(day_r, num_ONloans*num_participants),  do.call(c, time.repayments.i), do.call(c, ONrepayments.i), do.call(c, receiver.i), do.call(c, sender.i), rep(FXrate_r, num_ONloans*num_participants), rep(ONrate, num_ONloans*num_participants), rep(0, num_ONloans*num_participants), rep(2, num_ONloans*num_participants))

        output <- rbind(output.principal, output.repayment)

	trueloans <- cbind(output.principal, output.repayment, ONrates.d) 

	Output <- list(payoutput = output, trueloans = trueloans)

        return(Output)
}

#testON <- generate_overnightloans(num_participants, num_FX, ONrate, FXrate, FXrate+0.05, avgONsize, sdONsize,  20200101, 20200102)

